-------------------------------------------------------------------------------
-- Copyright (c) 2006 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
-------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 1.1
--  \   \        Filename: ddr1_controller.vhd
--  /   /        Date Last Modified: 5/11/06
-- /___/   /\    Date Created:
-- \   \  /  \
--  \___\/\___\
-- 
--Device: Virtex-5
--Purpose: Main DDR1 transaction control state machine. Handles all SDRAM
--         commands, except power-up initialization (initialization is done in
--         the PHY layer).
--Reference:
--    XAPP851
--Revision History:
--    Rev 1.0 - Internal release. Author: Toshihiko Moriyama. 4/29/06.
--    Rev 1.1 - External release. Added header. Deleted unused signal
--              chip_cnt(1:0). Changed ctrl_ddr_cke, ddr_cke_r to single bit.
--              5/11/06
--    Rev 1.2 - Fixed bug with CL=2.5 read-write spacing. rchiu. 6/28/06
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
library UNISIM;
use UNISIM.vcomponents.all;
use work.ddr1_parameters.all;

entity ddr1_controller is
	port(
		clk0			: in std_logic;
		refresh_clk		: in std_logic;
		rst				: in std_logic;
		af_addr			: in std_logic_vector(35 downto 0);
		af_empty		: in std_logic;
		phy_Dly_Slct_Done	: in std_logic;
		burst_length		: out std_logic_vector(2 downto 0);
		ctrl_af_RdEn		: out std_logic;
		ctrl_Wdf_RdEn		: out std_logic;
		ctrl_Dqs_Rst		: out std_logic;
		ctrl_Dqs_En			: out std_logic;
		ctrl_WrEn			: out std_logic;
		ctrl_RdEn			: out std_logic;
		ctrl_ddr_address	: out std_logic_vector((row_address -1) downto 0);
		ctrl_ddr_ba			: out std_logic_vector((bank_address -1) downto 0);
		ctrl_ddr_ras_L		: out std_logic;
		ctrl_ddr_cas_L		: out std_logic;
		ctrl_ddr_we_L		: out std_logic;
		ctrl_ddr_cs_L		: out std_logic_vector((no_of_cs -1) downto 0);
		ctrl_ddr_cke		: out std_logic
	    );
end ddr1_controller;

architecture arch of ddr1_controller is

constant IDLE                     : std_logic_vector(4 downto 0) := "00000";
constant LOAD_MODE_REG_ST         : std_logic_vector(4 downto 0) := "00001";
constant MODE_REGISTER_WAIT       : std_logic_vector(4 downto 0) := "00010";
constant PRECHARGE                : std_logic_vector(4 downto 0) := "00011";
constant PRECHARGE_WAIT           : std_logic_vector(4 downto 0) := "00100";
constant AUTO_REFRESH             : std_logic_vector(4 downto 0) := "00101";
constant AUTO_REFRESH_WAIT        : std_logic_vector(4 downto 0) := "00110";
constant ACTIVE                   : std_logic_vector(4 downto 0) := "00111";
constant ACTIVE_WAIT              : std_logic_vector(4 downto 0) := "01000";
constant FIRST_READ               : std_logic_vector(4 downto 0) := "01001";
constant BURST_READ               : std_logic_vector(4 downto 0) := "01010";
constant READ_WAIT                : std_logic_vector(4 downto 0) := "01011";
constant FIRST_WRITE              : std_logic_vector(4 downto 0) := "01100";
constant BURST_WRITE              : std_logic_vector(4 downto 0) := "01101";
constant WRITE_WAIT               : std_logic_vector(4 downto 0) := "01110";
constant WRITE_READ               : std_logic_vector(4 downto 0) := "01111";
constant READ_WRITE               : std_logic_vector(4 downto 0) := "10000";

constant    cs_h0               : std_logic_vector(3 downto 0)  := "0000";
constant    cs_h1               : std_logic_vector(3 downto 0)  := "0001";
constant    cs_h2               : std_logic_vector(3 downto 0)  := "0010";
constant    cs_h3               : std_logic_vector(3 downto 0)  := "0011";
constant    cs_h5               : std_logic_vector(3 downto 0)  := "0101";
constant    cs_h6               : std_logic_vector(3 downto 0)  := "0110";
constant    cs_h7               : std_logic_vector(3 downto 0)  := "0111";
constant    cs_hA               : std_logic_vector(3 downto 0)  := "1010";
constant    cs_hB               : std_logic_vector(3 downto 0)  := "1011";
constant    cs_hD               : std_logic_vector(3 downto 0)  := "1101";
constant    cs_hE               : std_logic_vector(3 downto 0)  := "1110";
constant    cs_hF               : std_logic_vector(3 downto 0)  := "1111";

signal ref_flag			: std_logic;
signal ref_flag_266		: std_logic;
signal ref_flag_266_r		: std_logic;
signal auto_ref			: std_logic;
signal next_state		: std_logic_vector(4 downto 0);
signal state			: std_logic_vector(4 downto 0);
signal state_r2			: std_logic_vector(4 downto 0);
signal state_r3			: std_logic_vector(4 downto 0);
signal row_addr_r		: std_logic_vector((row_address - 1) downto 0);
signal ddr_address_r1		: std_logic_vector((row_address - 1) downto 0);
signal ddr_ba_r1		: std_logic_vector((bank_address - 1) downto 0);
signal mrd_count		: std_logic;
signal rp_count			: std_logic_vector(2 downto 0);
signal rfc_count		: std_logic_vector(5 downto 0);
signal rcd_count		: std_logic_vector(2 downto 0);
signal ras_count		: std_logic_vector(3 downto 0);
signal wr_to_rd_count		: std_logic_vector(3 downto 0);
signal rd_to_wr_count		: std_logic_vector(3 downto 0);
signal rtp_count		: std_logic_vector(3 downto 0);
signal wtp_count		: std_logic_vector(3 downto 0);
signal refi_count		: std_logic_vector((max_ref_width - 1) downto 0);
--signal cas_count		: std_logic_vector(2 downto 0);
signal cas_check_count		: std_logic_vector(3 downto 0);
signal wrburst_cnt		: std_logic_vector(2 downto 0);
signal read_burst_cnt		: std_logic_vector(2 downto 0);
signal ctrl_WrEn_cnt		: std_logic_vector(2 downto 0);
signal rdburst_cnt		: std_logic_vector(2 downto 0);
--signal rc_count			: std_logic_vector(3 downto 0);
signal af_addr_r		: std_logic_vector(35 downto 0);
signal af_addr_r1		: std_logic_vector(35 downto 0);
signal wdf_rden_r		: std_logic;
signal wdf_rden_r2		: std_logic;
signal wdf_rden_r3		: std_logic;
signal wdf_rden_r4		: std_logic;
signal af_rden			: std_logic;
signal ddr_ras_r2		: std_logic;
signal ddr_cas_r2		: std_logic;
signal ddr_we_r2		: std_logic;
signal ddr_ras_r		: std_logic;
signal ddr_cas_r		: std_logic;
signal ddr_we_r			: std_logic;
signal ddr_ras_r3		: std_logic;
signal ddr_cas_r3		: std_logic;
signal ddr_we_r3		: std_logic;
signal idle_cnt			: std_logic_vector(3 downto 0);
signal conflict_resolved_r      : std_logic;
signal ddr_cs_r1		: std_logic_vector((no_of_cs - 1) downto 0);
signal ddr_cs_r			: std_logic_vector((no_of_cs - 1) downto 0);
signal ddr_cke_r		: std_logic;
signal burst_cnt		: std_logic_vector(3 downto 0);
signal burst_cnt_by2		: std_logic_vector(2 downto 0);
signal conflict_detect		: std_logic;
signal conflict_detect_r        : std_logic;
signal load_mode_reg		: std_logic_vector((row_address - 1) downto 0);
signal ext_mode_reg		: std_logic_vector((row_address - 1) downto 0);
signal CAS_LATENCY_VALUE        : std_logic_vector(3 downto 0);
signal BURST_LENGTH_VALUE       : std_logic_vector(2 downto 0);
signal REGISTERED_VALUE         : std_logic;
signal ECC_VALUE		: std_logic;
signal WR			: std_logic;
signal RD			: std_logic;
signal LMR			: std_logic;
signal PRE			: std_logic;
signal REF			: std_logic;
signal ACT			: std_logic;
signal WR_r			: std_logic;
signal RD_r			: std_logic;
signal LMR_r			: std_logic;
signal PRE_r			: std_logic;
signal REF_r			: std_logic;
signal ACT_r			: std_logic;
signal af_empty_r		: std_logic;
signal LMR_PRE_REF_ACT_cmd_r    : std_logic;
signal command_address		: std_logic_vector(2 downto 0);
signal zeroes			: std_logic_vector((row_address - col_ap_width) downto 0);
signal done_200us		: std_logic;
signal write_state		: std_logic;
signal read_state		: std_logic;
signal read_write_state         : std_logic;
signal burst_write_state        : std_logic;
signal first_write_state        : std_logic;
signal burst_read_state		: std_logic;
signal first_read_state         : std_logic;
signal burst_read_state_r2      : std_logic;
signal burst_read_state_r3      : std_logic;
signal first_read_state_r2      : std_logic;
signal read_write_state_r2      : std_logic;
signal read_write_state_r3      : std_logic;
signal ctrl_WrEn_r		: std_logic;
signal ctrl_WrEn_r1		: std_logic;
signal ctrl_Dqs_Rst_r		: std_logic;
signal ctrl_Dqs_Rst_r1		: std_logic;
signal ctrl_Dqs_En_r		: std_logic;
signal ctrl_Dqs_En_r1		: std_logic;
signal ctrl_RdEn_r		: std_logic;
signal ctrl_RdEn_r1		: std_logic;
signal ctrl_Wdf_RdEn_r		: std_logic;
signal ctrl_Wdf_RdEn_r1		: std_logic;

signal rd_to_wr_cas_delay       : std_logic;

begin

REGISTERED_VALUE <= '0';
CAS_LATENCY_VALUE	<= "0010" when (load_mode_reg(6 downto 4) = "110") else
			    '0' & load_mode_reg(6 downto 4) ;
BURST_LENGTH_VALUE	<= load_mode_reg(2 downto 0);
burst_length		<= burst_cnt(2 downto 0);
command_address		<= af_addr(34 downto 32);
zeroes			<= (others => '0');
ECC_VALUE		<= ecc_enable;

-- ADDED, RCHIU, 06/28/06
-- Fix for CL=2.5 read-write spacing. CL=2.5 has the same spacing from read
-- to write as for CL=3 (and not same spacing as CL=2). This signal is an
-- additive factor tacked on to an internal counter for forcing minimum delay
-- between reads and writes. =0 for CL=2, =1 for CL=2.5,3
rd_to_wr_cas_delay      <= '0' when (load_mode_reg(6 downto 4) = "010") else
                           '1';

burst_read_state	<= '1' when ((conflict_detect = '0') or (conflict_resolved_r = '1')) and (state = BURST_READ) and (RD = '1') else '0';
first_read_state	<= '1' when ((conflict_detect = '0') or (conflict_resolved_r = '1')) and (state = FIRST_READ) and (RD = '1') else '0';
read_state		<= burst_read_state or first_read_state;
read_write_state	<= write_state or read_state;
burst_write_state	<= '1' when ((conflict_detect = '0') or (conflict_resolved_r = '1')) and (state = BURST_WRITE) and (WR = '1') else '0';
first_write_state	<= '1' when ((conflict_detect = '0') or (conflict_resolved_r = '1')) and (state = FIRST_WRITE) and (WR = '1') else '0';
write_state		<= burst_write_state or first_write_state;

-- fifo control signals

ctrl_af_RdEn <= af_rden;

conflict_detect <= af_addr(35) and phy_Dly_Slct_Done and (not af_empty);

process(command_address, phy_Dly_Slct_Done, af_empty)
  begin
     WR  <= '0';
     RD  <= '0';
     LMR <= '0';
     PRE <= '0';
     REF <= '0';
     ACT <= '0';
     if((phy_Dly_Slct_Done = '1') and (af_empty = '0')) then
        case command_address is
		when "000" => LMR <= '1';
		when "001" => REF <= '1';
		when "010" => PRE <= '1';
		when "011" => ACT <= '1';
		when "100" => WR  <= '1';
		when "101" => RD  <= '1';
		when others => null;
        end case;
     end if;
  end process;

-- register address outputs
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
  if(rst = '1') then
     WR_r  <= '0';
     RD_r  <= '0';
     LMR_r <= '0';
     PRE_r <= '0';
     REF_r <= '0';
     ACT_r <= '0';
     af_empty_r <= '0';
     LMR_PRE_REF_ACT_cmd_r <= '0';
  else
     WR_r  <= WR;
     RD_r  <= RD;
     LMR_r <= LMR;
     PRE_r <= PRE;
     REF_r <= REF;
     ACT_r <= ACT;
     LMR_PRE_REF_ACT_cmd_r <= LMR or PRE or REF or ACT;
     af_empty_r <= af_empty;
  end if;
 end if;
end process;

-- register address outputs
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
   if(rst = '1') then
     af_addr_r           <= (others => '0');
     af_addr_r1          <= (others => '0');
     conflict_detect_r   <= '0';
     read_write_state_r2 <= '0';
     read_write_state_r3 <= '0';
     first_read_state_r2 <= '0';
     burst_read_state_r2 <= '0';
     burst_read_state_r3 <= '0';
  else
     af_addr_r		<= af_addr;
     af_addr_r1		<= af_addr_r;
     conflict_detect_r	<= conflict_detect;
     read_write_state_r2 <= read_write_state;
     read_write_state_r3 <= read_write_state_r2;
     first_read_state_r2 <= first_read_state;
     burst_read_state_r2 <= burst_read_state;
     burst_read_state_r3 <= burst_read_state_r2;
  end if;
 end if;
end process;

process (clk0)
begin
 if(clk0'event and clk0 = '1') then
   if(rst = '1') then
     load_mode_reg         <= load_mode_register((row_address-1) downto 0);
  elsif((state = LOAD_MODE_REG_ST) and (LMR_r = '1') and (af_addr_r((bank_address+row_address + col_ap_width -1) downto (col_ap_width + row_address))="00")) then
     load_mode_reg         <=  af_addr ((row_address-1) downto 0);
  end if;
 end if;
end process;

process (clk0)
begin
 if(clk0'event and clk0 = '1') then
   if(rst = '1') then
     ext_mode_reg         <= ext_load_mode_register((row_address-1) downto 0);
  elsif((state = LOAD_MODE_REG_ST) and (LMR_r = '1') and (af_addr_r((bank_address+row_address + col_ap_width -1) downto (col_ap_width + row_address))= "01")) then
     ext_mode_reg         <=  af_addr (row_address-1 downto 0);
  end if;
 end if;
end process;

-- mrd count
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
       mrd_count <= '0';
   elsif (state = LOAD_MODE_REG_ST) then
       mrd_count <= mrd_count_value;
   elsif (mrd_count /= '0') then
      mrd_count <= '0';
   else
     mrd_count <= '0';
  end if;
 end if;
end process;

-- rp count
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
       rp_count(2 downto 0) <= "000";
   elsif (state = PRECHARGE) then
       rp_count(2 downto 0) <= rp_count_value;
   elsif (rp_count(2 downto 0) /= "000") then
       rp_count(2 downto 0) <= rp_count(2 downto 0) - 1;
   else
     rp_count(2 downto 0) <= "000";
  end if;
 end if;
end process;

-- rfc count
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
       rfc_count(5 downto 0) <= "000000";
   elsif (state = AUTO_REFRESH) then
       rfc_count(5 downto 0) <= rfc_count_value;
   elsif (rfc_count(5 downto 0) /= "000000") then
       rfc_count(5 downto 0) <= rfc_count(5 downto 0) - 1;
   else
     rfc_count(5 downto 0) <= "000000";
  end if;
 end if;
end process;

-- rcd count - 20ns
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
       rcd_count(2 downto 0) <= "000";
   elsif (state = ACTIVE) then
       rcd_count(2 downto 0) <= rcd_count_value;
   elsif (rcd_count(2 downto 0) /= "000") then
       rcd_count(2 downto 0) <= rcd_count(2 downto 0) - 1;
   else
       rcd_count(2 downto 0) <=  "000";
  end if;
 end if;
end process;

-- ras count - active to precharge
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
       ras_count(3 downto 0) <= "0000";
   elsif (state = ACTIVE) then
       ras_count(3 downto 0) <= ras_count_value;
   elsif (ras_count(3 downto 1) = "000") then
       if (ras_count(0) /= '0') then
              ras_count(0) <= '0';
       end if;
   else
       ras_count(3 downto 0) <= ras_count(3 downto 0) - 1;
  end if;
 end if;
end process;

--AL+BL/2+TRTP-2
-- rtp count - read to precharge
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
       rtp_count(3 downto 0) <= "0000";
   elsif (read_state = '1') then
       rtp_count(2 downto 0) <= trtp_count_value ;
   elsif (rtp_count(3 downto 1) = "000") then
       if (rtp_count(0) /= '0') then
              rtp_count(0) <= '0';
       end if;
   else
       rtp_count(3 downto 0) <= rtp_count(3 downto 0) - 1;
  end if;
 end if;
end process;

-- WL+BL/2+TWR
-- wtp count - write to precharge
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
       wtp_count(3 downto 0) <= "0000";
   elsif (write_state = '1') then
       wtp_count(2 downto 0) <= twr_count_value  ;
   elsif (wtp_count(3 downto 1) = "000") then
       if (wtp_count(0) /= '0') then
              wtp_count(0) <= '0';
       end if;
   else
       wtp_count(3 downto 0) <= wtp_count(3 downto 0) - 1;
  end if;
 end if;
end process;

-- write to read counter
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
       wr_to_rd_count(3 downto 0) <= "0000";
   elsif (write_state = '1') then
       wr_to_rd_count(2 downto 0) <= twtr_count_value;
   elsif (wr_to_rd_count(3 downto 0) /= "0000") then
       wr_to_rd_count(3 downto 0) <= wr_to_rd_count(3 downto 0) - 1;
   else
     wr_to_rd_count(3 downto 0) <= "0000";
  end if;
 end if;
end process;

-- read to write counter
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
      rd_to_wr_count(3 downto 0) <= "0000";
   elsif (read_state = '1') then
      -- MODIFIED, RCHIU, 06/28/06
      -- Fixed CL=2.5 read-write spacing bug
      rd_to_wr_count(3 downto 0) <=  registered + burst_cnt + rd_to_wr_cas_delay;
--      rd_to_wr_count(3 downto 0) <=  registered + burst_cnt + CAS_LATENCY_VALUE(0);
   elsif (rd_to_wr_count(3 downto 0) /= "0000") then
      rd_to_wr_count(3 downto 0) <= rd_to_wr_count(3 downto 0) - 1;
   else
     rd_to_wr_count(3 downto 0) <= "0000";
  end if;
 end if;
end process;

-- auto refresh interval counter in refresh_clk domain
process (refresh_clk)
begin
 if(refresh_clk'event and refresh_clk = '1') then
   if(rst = '1') then
      refi_count <= (others => '0');
   elsif (refi_count = max_ref_cnt ) then
      refi_count <= (others => '0');
   else
      refi_count <= refi_count + 1;
  end if;
 end if;
end process;

 ref_flag <= '1' when ((refi_count = max_ref_cnt) and (done_200us = '1') ) else
	    '0';

--200us counter for cke
-- Commented for later use
--process (refresh_clk)
--begin
-- if(refresh_clk'event and refresh_clk = '1') then
--  if (rst = '1') then
--    cke_200us_cnt <= "11011";
--  elsif (refi_count(max_ref_width-1  downto  0) =  max_ref_cnt) then
--    cke_200us_cnt  <=  cke_200us_cnt - 1;
--  else
--    cke_200us_cnt  <= cke_200us_cnt;
--  end if;
-- end if;
--end process;

-- refresh detect in 266 MHz clock
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
      ref_flag_266   <= '0';
      ref_flag_266_r <= '0';
      done_200us <= '0';
   else
      ref_flag_266   <= ref_flag;
      ref_flag_266_r <= ref_flag_266;
--      if (done_200us = '0' and (cke_200us_cnt = "00000")) then --comment should be removed later
	done_200us <= '1';
--      end if;
  end if;
 end if;
end process;

--refresh flag detect
--auto_ref high indicates auto_refresh requirement
--auto_ref is held high until auto refresh command is issued.
process(clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
       auto_ref <= '0';
    elsif (ref_flag_266 = '1' and ref_flag_266_r = '0') then
       auto_ref <= '1';
    elsif (state = AUTO_REFRESH) then
       auto_ref <= '0';
    else
       auto_ref <= auto_ref;
  end if;
 end if;
end process;


burst_cnt           <= "0010" when (BURST_LENGTH_VALUE = "010") else
                       "0100" when (BURST_LENGTH_VALUE = "011") else
		       "0001";
burst_cnt_by2	     <= "001" when (BURST_LENGTH_VALUE = "010") else
                       "010" when (BURST_LENGTH_VALUE = "011") else
		       "000";

-- write burst count
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
       wrburst_cnt(2 downto 0) <= "000";
   elsif (write_state = '1') then
       wrburst_cnt(2 downto 0) <= burst_cnt(2 downto 0);
   elsif (wrburst_cnt(2 downto 0) /= "000") then
       wrburst_cnt(2 downto 0) <= wrburst_cnt(2 downto 0) - 1;
   else wrburst_cnt(2 downto 0) <= "000";
  end if;
 end if;
end process;

-- read burst count for state machine
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
       read_burst_cnt(2 downto 0) <= "000";
   elsif (read_state = '1') then
       read_burst_cnt(2 downto 0) <= burst_cnt(2 downto 0);
   elsif (read_burst_cnt(2 downto 0) /= "000") then
       read_burst_cnt(2 downto 0) <= read_burst_cnt(2 downto 0) - 1;
   else read_burst_cnt(2 downto 0) <= "000";
  end if;
 end if;
end process;

-- count to generate write enable to the data path
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
       ctrl_WrEn_cnt(2 downto 0) <= "000";
   elsif (wdf_rden_r = '1') then
       ctrl_WrEn_cnt(2 downto 0) <= burst_cnt(2 downto 0);
   elsif (ctrl_WrEn_cnt(2 downto 0) /= "000") then
       ctrl_WrEn_cnt(2 downto 0) <= ctrl_WrEn_cnt(2 downto 0) -1;
   else
       ctrl_WrEn_cnt(2 downto 0) <= "000";
  end if;
 end if;
end process;

--write enable to data path
process (ctrl_WrEn_cnt)
begin
     if (ctrl_WrEn_cnt(2 downto 0) /= "000") then
        ctrl_WrEn_r <= '1';
     else
        ctrl_WrEn_r <= '0';
     end if;
end process;

process(clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
	ctrl_WrEn_r1 <= '0';
    else
	ctrl_WrEn_r1 <= ctrl_WrEn_r;
    end if;
 end if;
end process;

ctrl_WrEn <= ctrl_WrEn_r1 when (REGISTERED_VALUE = '1') else
	     ctrl_WrEn_r;


-- DQS reset to data path
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
      ctrl_Dqs_Rst_r <= '0';
   elsif (first_write_state = '1') then
      ctrl_Dqs_Rst_r <= '1';
   else
      ctrl_Dqs_Rst_r <= '0';
  end if;
 end if;
end process;

process(clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
	ctrl_Dqs_Rst_r1 <= '0';
    else
	ctrl_Dqs_Rst_r1 <= ctrl_Dqs_Rst_r;
    end if;
 end if;
end process;

ctrl_Dqs_Rst <= ctrl_Dqs_Rst_r1 when (REGISTERED_VALUE = '1') else
		ctrl_Dqs_Rst_r;

-- DQS enable to data path
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
      ctrl_Dqs_En_r <= '0';
   elsif ((write_state = '1') or (wrburst_cnt /= "000")) then
      ctrl_Dqs_En_r <= '1';
   else
      ctrl_Dqs_En_r <= '0';
  end if;
 end if;
end process;

process(clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
	ctrl_Dqs_En_r1 <= '0';
    else
	ctrl_Dqs_En_r1 <= ctrl_Dqs_En_r;
    end if;
 end if;
end process;

ctrl_Dqs_En <= ctrl_Dqs_En_r1 when (REGISTERED_VALUE = '1') else
	       ctrl_Dqs_En_r;


-- read_wait/write_wait to idle count
-- the state machine waits for 15 clock cycles in the write wait state for any wr/rd commands
-- to be issued. If no commands are issued in 15 clock cycles, the statemachine issues
--  enters the idle state and stays in the idle state until an auto refresh is required.

process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
       idle_cnt(3 downto 0) <= "0000";
   elsif (read_write_state = '1') then
       idle_cnt(3 downto 0) <= "1111" ;
   elsif (idle_cnt(3 downto 0) /= "0000") then
       idle_cnt(3 downto 0) <= idle_cnt(3 downto 0) - 1;
   else idle_cnt(3 downto 0) <= "0000";
  end if;
 end if;
end process;

process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
       cas_check_count(3 downto 0) <= "0000";
   elsif (first_read_state_r2 = '1')  then
          cas_check_count(3 downto 0) <= (CAS_LATENCY_VALUE - 1);
   elsif (cas_check_count(3 downto 0) /= "0000") then
       cas_check_count(3 downto 0) <= cas_check_count(3 downto 0) - 1;
   else
     cas_check_count(3 downto 0) <= "0000";
  end if;
 end if;
end process;

process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if (rst = '1') then
      rdburst_cnt(2 downto 0) <= "000";
      ctrl_RdEn_r <= '0';
   elsif ((cas_check_count = "0001") and (burst_read_state_r3 = '0')) then
      rdburst_cnt(2 downto 0) <= burst_cnt(2 downto 0);
      ctrl_RdEn_r <= '1';
   elsif(burst_read_state_r3 = '1') then
      if(burst_cnt = "0100") then
	rdburst_cnt(2 downto 0) <= CAS_LATENCY_VALUE(2 downto 0) + burst_cnt_by2;
      elsif (burst_cnt = "010") then
        rdburst_cnt(2 downto 0) <= CAS_LATENCY_VALUE(2 downto 0);
      else
	rdburst_cnt(2 downto 0) <= CAS_LATENCY_VALUE(2 downto 0) - burst_cnt(2 downto 0);
      end if;
      ctrl_RdEn_r <= '1';
   elsif (rdburst_cnt(2 downto 0) /= "000") then
      rdburst_cnt(2 downto 0) <= rdburst_cnt(2 downto 0) - '1';
      if(rdburst_cnt = "001") then
        ctrl_RdEn_r <= '0';
      end if;
   else
     rdburst_cnt(2 downto 0) <= "000";
  end if;
 end if;
end process;

process(clk0)
begin
 if(clk0'event and clk0 = '1') then
    if (rst = '1') then
	ctrl_RdEn_r1 <= '0';
    else
	ctrl_RdEn_r1 <= ctrl_RdEn_r;
    end if;
 end if;
end process;

ctrl_RdEn <= ctrl_RdEn_r1 when (REGISTERED_VALUE = '1') else
	     ctrl_RdEn_r;

-- write address FIFO read enable signals

af_rden <= '1' when ((read_write_state = '1') or ((state = MODE_REGISTER_WAIT) and LMR_r = '1' and (mrd_count = '0')) or
                     ((state = PRECHARGE )and PRE_r = '1') or ((state = AUTO_REFRESH) and REF_r = '1')
                     or ((state = ACTIVE )and ACT_r = '1')) else '0';

-- write data fifo read enable
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if (rst = '1')  then
       wdf_rden_r  <= '0';
   elsif (write_state = '1') then -- place holder for burst_write
       wdf_rden_r  <= '1';
   else
     wdf_rden_r  <= '0';
  end if;
 end if;
end process;

process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if (rst = '1') then
       wdf_rden_r2 <= '0';
       wdf_rden_r3 <= '0';
       wdf_rden_r4 <= '0';
   else
       wdf_rden_r2 <= wdf_rden_r;
       wdf_rden_r3 <= wdf_rden_r2;
       wdf_rden_r4 <= wdf_rden_r3;
  end if;
 end if;
end process;

-- Read enable to the data fifo

process (burst_cnt, wdf_rden_r, wdf_rden_r2, wdf_rden_r3, wdf_rden_r4)
begin
   if (burst_cnt = "001") then
       ctrl_Wdf_RdEn_r<= (wdf_rden_r ) ;
   elsif (burst_cnt = "010") then
       ctrl_Wdf_RdEn_r<= (wdf_rden_r or wdf_rden_r2) ;
   elsif (burst_cnt = "100") then
       ctrl_Wdf_RdEn_r<= (wdf_rden_r or wdf_rden_r2 or wdf_rden_r3 or wdf_rden_r4) ;
   else
       ctrl_Wdf_RdEn_r<= '0';
   end if;
end process;

process (clk0)
begin
 if(clk0'event and clk0 = '1') then
   if(rst = '1') then
	ctrl_Wdf_RdEn_r1 <= '0';
   else
	ctrl_Wdf_RdEn_r1 <= ctrl_Wdf_RdEn_r;
   end if;
 end if;
end process;

ctrl_Wdf_RdEn <= ctrl_Wdf_RdEn_r1 when (REGISTERED_VALUE = '1') else
		 ctrl_Wdf_RdEn_r;

process (clk0)
begin
 if(clk0'event and clk0 = '1') then
   if(rst = '1') then
      state <= IDLE;
  else
     state <= next_state;
  end if;
 end if;
end process;

--Main control state machine
process (ACT_r, LMR_PRE_REF_ACT_cmd_r, LMR_r, RD
	 , RD_r, REF_r, WR, WR_r, auto_ref
	 , conflict_detect, conflict_detect_r
	 , conflict_resolved_r, idle_cnt, mrd_count, ras_count, rcd_count
	 , rd_to_wr_count, read_burst_cnt, rfc_count, rp_count
	 , rtp_count, state, wr_to_rd_count, wrburst_cnt
	 , wtp_count,burst_cnt, af_empty_r)
begin

   next_state <= state;
   case state is
	when IDLE =>
	if ((conflict_detect_r = '1' or LMR_PRE_REF_ACT_cmd_r = '1' or auto_ref = '1') and ras_count = "0000") then
	   next_state <= PRECHARGE;
	elsif ((WR_r= '1'  or RD_r= '1') and (ras_count = "0000")) then
	   next_state <= ACTIVE;
	end if;

        when LOAD_MODE_REG_ST  => next_state <= MODE_REGISTER_WAIT;

	when MODE_REGISTER_WAIT =>
        if (mrd_count = '0') then
          next_state <= IDLE;
        else
          next_state <= MODE_REGISTER_WAIT;
        end if;

	when PRECHARGE  => next_state <= PRECHARGE_WAIT;


	when PRECHARGE_WAIT =>
        if (rp_count = "000") then
           if ((auto_ref or REF_r) = '1') then
              next_state <= AUTO_REFRESH;
	   elsif (LMR_r = '1') then
              next_state <= LOAD_MODE_REG_ST;
	   elsif ((conflict_detect_r or ACT_r) = '1') then
              next_state <= ACTIVE;
           else
              next_state <= IDLE;
           end if;
        else
           next_state <= PRECHARGE_WAIT;
        end if;

	when AUTO_REFRESH => next_state <= AUTO_REFRESH_WAIT;

	when AUTO_REFRESH_WAIT =>
        if ( (rfc_count = "00001") and (conflict_detect_r = '1') ) then
           next_state <= ACTIVE;
        elsif (rfc_count = "00001") then
           next_state <= IDLE;
        else
           next_state <= AUTO_REFRESH_WAIT;
        end if;

	when ACTIVE  => next_state <= ACTIVE_WAIT;


	when ACTIVE_WAIT  =>
        if (rcd_count = "000") then
	   if(WR = '1') then
             next_state <= FIRST_WRITE;
           elsif (RD = '1') then
             next_state <= FIRST_READ;
           else
             next_state <= IDLE;
	   end if;
	else
	  next_state <= ACTIVE_WAIT;
	end if;

	when FIRST_WRITE  =>
	      if((((conflict_detect = '1') and (conflict_resolved_r ='0')) or (auto_ref = '1')) or RD = '1') then
		 next_state <= WRITE_WAIT;
	      elsif((burst_cnt = "001") and (WR = '1')) then
		  next_state <= BURST_WRITE;
	      else
		  next_state <= WRITE_WAIT;
	      end if;

	when BURST_WRITE =>
	      if((((conflict_detect = '1') and (conflict_resolved_r = '0')) or (auto_ref = '1')) or (RD = '1')) then
		 next_state <= WRITE_WAIT;
	      elsif((burst_cnt = "001") and (WR = '1')) then
		  next_state <= BURST_WRITE;
	      else
		  next_state <= WRITE_WAIT;
	      end if;

	when WRITE_WAIT =>
        if (((conflict_detect = '1') and (conflict_resolved_r = '0')) or (auto_ref = '1')) then
	   if ((wtp_count = "0000") and (ras_count = "0000")) then
             next_state <= PRECHARGE;
           else
             next_state <= WRITE_WAIT;
	   end if;
        elsif (RD = '1') then
           next_state <= WRITE_READ;
        elsif ((WR ='1') and (wrburst_cnt = "010")) then
           next_state <= BURST_WRITE;
        elsif (idle_cnt = "0000") then
           next_state <= PRECHARGE;
	else
           next_state <= WRITE_WAIT;
	end if;

	when WRITE_READ =>
        if (wr_to_rd_count = "0000") then
           next_state <= FIRST_READ;
        else
           next_state <= WRITE_READ;
        end if;

	when FIRST_READ   =>
	      if((((conflict_detect = '1') and (conflict_resolved_r = '0')) or (auto_ref = '1')) or (WR = '1')) then
		 next_state <= READ_WAIT;
	      elsif((burst_cnt = "001") and (RD = '1')) then
		  next_state <= BURST_READ;
	      else
		  next_state <= READ_WAIT;
	      end if;

	when BURST_READ  =>
	      if((((conflict_detect = '1') and (conflict_resolved_r = '0'))or (auto_ref = '1')) or (WR = '1')) then
		 next_state <= READ_WAIT;
	      elsif((burst_cnt = "001") and (RD = '1')) then
		  next_state <= BURST_READ;
	      else
		  next_state <= READ_WAIT;
	      end if;

	when READ_WAIT  =>
        if (((conflict_detect = '1') and (conflict_resolved_r = '0')) or (auto_ref = '1')) then
           if(rtp_count = "0000" and ras_count = "0000") then
             next_state <= PRECHARGE;
           else
             next_state <= READ_WAIT;
	   end if;
        elsif (WR = '1') then
           next_state <= READ_WRITE;
        elsif ((RD = '1') and (read_burst_cnt <= "010")) then
	   if(af_empty_r = '1') then
	     next_state <= FIRST_READ;
	   else
             next_state <= BURST_READ;
	   end if;
        elsif (idle_cnt = "0000") then
           next_state <= PRECHARGE;
        else
           next_state <= READ_WAIT;
        end if;


	when READ_WRITE  =>
        if (rd_to_wr_count = "0000") then
           next_state <= FIRST_WRITE;
        else
           next_state <= READ_WRITE;
        end if;

	when others => next_state <= IDLE;

   end case;
end process;

--register command outputs
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
      state_r2 <= "00000";
      state_r3 <= "00000";
   else
      state_r2 <= state;
      state_r3 <= state_r2;
  end if;
 end if;
end process;

-- commands to the memory
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
      ddr_ras_r <= '1';
   elsif ((state = LOAD_MODE_REG_ST) or (state = PRECHARGE) or (state = ACTIVE) or (state = AUTO_REFRESH)) then
      ddr_ras_r <= '0';
   else ddr_ras_r <= '1';
  end if;
 end if;
end process;

-- commands to the memory
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
      ddr_cas_r <= '1';
   elsif ((state = LOAD_MODE_REG_ST)  or (read_write_state = '1') or (state = AUTO_REFRESH)) then
      ddr_cas_r <= '0';
   elsif ((state = ACTIVE_WAIT)) then
      ddr_cas_r <= '1';
   else
      ddr_cas_r <= '1';
  end if;
 end if;
end process;

-- commands to the memory
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
      ddr_we_r <= '1';
   elsif ((state = LOAD_MODE_REG_ST) or (state = PRECHARGE) or (write_state = '1')) then
      ddr_we_r <= '0';
   else ddr_we_r <= '1';
  end if;
 end if;
end process;

--register commands to the memory
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
      ddr_ras_r2 <= '1';
      ddr_cas_r2 <= '1';
      ddr_we_r2 <= '1';
   else
      ddr_ras_r2  <= ddr_ras_r;
      ddr_cas_r2  <= ddr_cas_r;
      ddr_we_r2   <= ddr_we_r;
  end if;
 end if;
end process;

--register commands to the memory
process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if (rst = '1') then
	ddr_ras_r3 <= '1';
	ddr_cas_r3 <= '1';
	ddr_we_r3  <= '1';
   else
	ddr_ras_r3  <= ddr_ras_r2;
	ddr_cas_r3  <= ddr_cas_r2;
	ddr_we_r3   <= ddr_we_r2;
  end if;
 end if;
end process;

process (clk0)
begin
 if(clk0'event and clk0 = '1') then
   if(rst = '1') then
       row_addr_r(row_address-1 downto 0) <= (others => '0');
  else
       row_addr_r(row_address-1 downto 0) <= af_addr((row_address + col_ap_width)-1 downto col_ap_width);
  end if;
 end if;
end process;

-- chip enable generation logic
process(clk0)
 begin
  if (clk0='1' and clk0'event) then
      if (rst = '1')  then
          ddr_cs_r((no_of_cs-1) downto 0) <=  (others => '0');
      else
          if (af_addr_r((chip_address + bank_address +row_address + (col_ap_width-1)) downto (bank_address + row_address + col_ap_width)) =
	      cs_h0((chip_address - 1) downto 0)) then
                       ddr_cs_r((no_of_cs-1) downto  0) <=  cs_hE((no_of_cs-1) downto  0);
          elsif (af_addr_r((chip_address + bank_address + row_address + (col_ap_width-1)) downto (bank_address +row_address + col_ap_width)) =
		 cs_h1((chip_address - 1) downto 0))   then
                     ddr_cs_r((no_of_cs-1) downto 0) <=  cs_hD((no_of_cs-1) downto  0);
          elsif (af_addr_r((chip_address + bank_address +row_address + (col_ap_width-1)) downto (bank_address + row_address + col_ap_width)) =
		  cs_h2((chip_address - 1) downto 0)) then
                     ddr_cs_r((no_of_cs-1) downto 0) <= cs_hB((no_of_cs-1) downto  0);
         elsif (af_addr_r((chip_address + bank_address +row_address + (col_ap_width-1)) downto (bank_address + row_address + col_ap_width)) =
		    cs_h3((chip_address - 1) downto 0)) then
		         ddr_cs_r((no_of_cs-1) downto 0) <=  cs_h7((no_of_cs-1) downto  0);
         else
                     ddr_cs_r((no_of_cs-1) downto 0) <=  cs_hF((no_of_cs-1) downto  0);
         end if;
      end if;
  end if;
end process;

process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
      ddr_address_r1 <= (others => '0');
   elsif ((state_r2 = ACTIVE)) then
      ddr_address_r1 <= row_addr_r;
   elsif (read_write_state_r2 = '1') then
      ddr_address_r1 <= af_addr_r(row_address-1 downto 0) and add_const4((row_address-1) downto 0); -- Auto Precharge option is disabled
   elsif (state_r2 = PRECHARGE) then
      if(PRE_r = '1') then
        ddr_address_r1 <= af_addr_r(row_address-1 downto 0);
      else
        ddr_address_r1 <= add_const1((row_address-1) downto 0);--X"0400";
      end if;
   elsif (state_r2 = LOAD_MODE_REG_ST)then
      ddr_address_r1 <= af_addr_r(row_address-1 downto 0);
   else
      ddr_address_r1 <= add_const3((row_address-1) downto 0);--X"0000";
  end if;
 end if;
end process;

process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
      ddr_ba_r1(bank_address-1 downto 0) <= (others => '0');
   elsif ((state_r2 = ACTIVE) or (state_r2 = LOAD_MODE_REG_ST) or ((state_r2 = PRECHARGE) and PRE_r = '1')) then
      ddr_ba_r1(bank_address-1 downto 0) <= af_addr((bank_address+row_address + col_ap_width)-1 downto (col_ap_width + row_address));
   else
      ddr_ba_r1(bank_address-1 downto 0) <= ddr_ba_r1(bank_address-1 downto 0);
  end if;
 end if;
end process;

process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
      ddr_cs_r1 <= (others => '0');
    elsif (state_r3 = auto_refresh ) then
      ddr_cs_r1 <= (others => '0');
    elsif ((state_r3 = active ) or (state_r3 = load_mode_reg_st) or (state_r3 = precharge_wait )) then
      ddr_cs_r1 <= ddr_cs_r;
    else
      ddr_cs_r1 <= ddr_cs_r1;
    end if;
  end if;
end process;

process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
      conflict_resolved_r <= '0';
   else
      if ((state = PRECHARGE_WAIT)  and (conflict_detect_r= '1')) then
        conflict_resolved_r  <= '1';
      elsif(af_rden = '1') then
        conflict_resolved_r  <= '0';
      end if;
  end if;
 end if;
end process;

process (clk0)
begin
 if(clk0'event and clk0 = '1') then
    if(rst = '1') then
       ddr_cke_r<= '0';
   else
      if(done_200us = '1') then
	ddr_cke_r<= '1';
      end if;
   end if;
 end if;
end process;

 ctrl_ddr_address(row_address-1 downto 0)  <= ddr_address_r1(row_address-1 downto 0);
 ctrl_ddr_ba (bank_address-1 downto 0)     <= ddr_ba_r1(bank_address-1 downto 0);
 ctrl_ddr_ras_L <= ddr_ras_r2;
 ctrl_ddr_cas_L <= ddr_cas_r2;
 ctrl_ddr_we_L  <= ddr_we_r2;
 ctrl_ddr_cs_L <= (others => '0');
 ctrl_ddr_cke  <= ddr_cke_r;

end arch;
